% satisfiability proof
:- dec_p_type(dangerousPermMissing).
dangerousPermMissing :-
  dec(S,system) & dec(SystemPerm,set(perm)) & 
  dec(P,perm) & dec(A,idApp) &

  state(S,St) & dec(St,state) &
  apps(St,Apps) & dec(Apps,set(idApp)) &

  validstate(SystemPerm,S) &
  pl(P,dangerous) &
  permExists(SystemPerm,P,S) &
  A in Apps &
  getManifestForApp(A,S,M) & dec(M,manifest) &
  use(M,SP) & dec(SP,set(perm)) &
  P in SP &
  neg(appHasPermission(SystemPerm,A,P,S)).

% satisfiability proof
:- dec_p_type(delegateGrantPRevoke).
delegateGrantPRevoke :-
  dec([S,S1,S2,S3],system) & dec(P,perm) & dec([A,A_],idApp) &
  dec([IC,IC_],iCmp) & dec([C,C_],cmp) & dec(U,uri) &
  dec(CP,cProvider) & dec(SystemPerm,set(perm)) &

  state(S,St) & dec(St,state) &
  running(St,RR) & dec(RR,rel(iCmp,cmp)) &

  validstate(SystemPerm,S) &

% not necessary due to grant(SystemPerm,A,P,S,S1)
% i.e. in grant we have pre_grant included
%  pre_grant(SystemPerm,A,P,S) &
  getManifestAndAppFromCmp(C,S,A,M1) & dec(M1,manifest) &
  getManifestAndAppFromCmp(C_,S,A_,M2) & dec(M2,manifest) &
  applyTo(RR,IC,C) &
  applyTo(RR,IC_,C_) &
  canGrant(CP,U,S) &
  existsRes(CP,U,S) &
  expC(CP,boolc(true)) &
  readE(CP,permc(P)) &

  grant(SystemPerm,A,P,S,S1) &
  grantP(SystemPerm,IC,CP,A_,U,read,S1,S2) &
  revoke(P,A,S2,S3) &

% instead of getting the answer of running read
% we check its precondition holds in S3
  pre_read(SystemPerm,IC_,CP,U,S3).

% ifPermThenGranted
% sequence
% takes too long due to appHasPermission

% revokeCanStart
% sequence
% takes too long due to canStart

% TYPES_property1
:- dec_p_type(cannotAutoGrantWithoutGroup).
cannotAutoGrantWithoutGroup :-
  dec([S,S_],system) & dec(SystemPerm,set(perm)) & 
  dec(A,idApp) & dec(P,perm) & dec(OG,optg) & dec(G,idGrp) &
  dec(GR,rel(idApp,set(idGrp))) & dec(GA,set(idGrp)) &
  dec(St,state) &

  state(S,St) & 
  maybeGrp(P,OG) & 
  grantedPermGroups(St,GR) &
  neg(
    pl(P,dangerous) &
    OG = grp(G) &
    applyTo(GR,A,GA) & 
    G nin GA
    implies neg(grantAuto(SystemPerm,P,A,S,S_))
  ).

% satisfiability proof
% TYPES_property2
:- dec_p_type(execAutoGrantWithoutIndividualPerms).
execAutoGrantWithoutIndividualPerms :-
  dec(S,system) & dec(P,perm) & dec(G,idGrp) & dec(A,idApp) &
  dec(SystemPerm,set(perm)) &

  state(S,St) & dec(St,state) &
  perms(St,PR) & dec(PR,rel(idApp,set(perm))) &

  validstate(SystemPerm,S) &
  applyTo(PR,A,PermsA) & dec(PermsA,set(perm)) &
  nexists(P_ in PermsA,[MG],dec(P_,perm) & 
    MG = grp(G),
    maybeGrp(P_,MG) & dec(MG,optg)
  ) &
  pl(P,dangerous) &
  maybeGrp(P,grp(G)) &
  pre_grantAuto(SystemPerm,P,A,S).


% satisfiability proof
% TYPES_property3
:- dec_p_type(dangerousPermissionAutoGranted).
dangerousPermissionAutoGranted :-
  dec([S,S_],system) & dec(SystemPerm,set(perm)) & 
  dec(A,idApp) & dec(M,manifest) & dec(C,cert) & dec(RS,set(res)) &
  dec([PDang,PNorm],perm) & dec(G,idGrp) &
  dec(St,state) & dec(PM,set(perm)) & dec(InitVal,val) &

  use(M,PM) & 

  validstate(SystemPerm,S) &
  permExists(SystemPerm,PDang,S_) &
  pl(PDang,dangerous) &
  pl(PNorm,normal) &
  PNorm in PM &
  PDang in PM &
  maybeGrp(PNorm,grp(G)) &
  maybeGrp(PDang,grp(G)) &
  install(SystemPerm,InitVal,A,M,C,RS,S,S_) &  % install is executed between S, S_
  pre_grantAuto(SystemPerm,PDang,A,S_).        % and S_ satisfies the precondition
                                               % of grantAuto

% es revokePermGroup o revokeGroup
% revokePermGroup no hace nada con perms !!!
% tampoco dice que las otras componentes de S no cambian !!!
% TYPES_property4
:- dec_p_type(revokeGroupRevokesIndividualPerms).
revokeGroupRevokesIndividualPerms :-
  dec([S,S_],system) & dec(SystemPerm,set(perm)) & 
  dec(A,idApp) & dec(G,idGrp) & dec(SP,set(perm)) &
  dec(St_,state) & dec(Perms_,rel(idApp,set(perm))) & 

  state(S_,St_) & 
  perms(St_,Perms_) &
  applyTo(Perms_,A,SP) &

  neg(
    revokeGroup(G,A,S,S_)
    implies 
    neg(
      exists(P in SP,[G1],dec(P,perm) & 
        G1 = G,
        maybeGrp(P,grp(G1)) & dec(G1,idGrp)
      )
    )
  ).

% TYPES_property5
:- dec_p_type(notVerifiedOldAppCantReceive).
notVerifiedOldAppCantReceive :-
  dec([S,S_],system) & dec(SystemPerm,set(perm)) & 
  dec(I,intent) & dec(IC,iCmp) & dec(A,idApp) &
  dec(VulnerableSdk,int) & dec(IntentActionType,rel(intent,pType)) &
  dec(St,state) & dec(AV,set(idApp)) &

  state(S,St) &
  alreadyVerified(St,AV) &

  notDupSysApp(S) & notDupApp(S) & statesConsistency1(S) &
  neg(
    isOldApp(VulnerableSdk,A,S) &
    A nin AV
    implies 
      neg(receiveIntent3(SystemPerm,VulnerableSdk,IntentActionType,I,IC,A,S,S_))
  ).


% TYPES_property6
% satisfiability proof
:- dec_p_type(sacProtectedWithNormalPerms).
sacProtectedWithNormalPerms :-
  dec(S,system) & dec(Access_internet,sacall) &
  dec(C,cmp) & dec(IC,iCmp) & dec(SystemPerm,set(perm)) &

  state(S,St) & dec(St,state) &
  running(St,RR) & dec(RR,rel(iCmp,cmp)) &

  validstate(SystemPerm,S) &
  permSAC(SystemPerm,Access_internet,Perms) & dec(Perms,set(perm)) &
  getManifestAndAppFromCmp(C,S,A,M1) & dec(A,idApp) & dec(M1,manifest) &
  getManifestForApp(A,S,M2) & dec(M2,manifest) &
  use(M2,SP) & dec(SP,set(perm)) &
  foreach(P in Perms,pl(P,normal) & P in SP & dec(P,perm)) &
  applyTo(RR,IC,C) & 
  pre_call(SystemPerm,IC,Access_internet,S).
  

:- dec_p_type(eavesdropping1).
eavesdropping1 :-
  dec(IC,iCmp) & dec(I,intent) & dec(Cmp,cmp) &
  dec([A,A_],idApp) & dec([C,C_],cert) &
  dec(S,system) & dec(SystemPerm,set(perm)) & 
  dec(VulnerableSdk,int) & dec(IntentActionType,rel(intent,pType)) &

  state(S,St) & dec(St,state) &
  running(St,RR) & dec(RR,rel(iCmp,cmp)) &
  sentIntents(St,SSInt) & dec(SSInt,set([iCmp,intent])) &
  environment(S,E) & dec(E,environment) &
  systemImage(E,SSI) & dec(SSI,set(sysImgApp)) &

% axiom onlyDangInPerms
  perms(St,PR) & dec(PR,rel(idApp,set(perm))) &
  foreach([X,SP] in PR,dec(X,idApp) & dec(SP,set(perm)) &
    foreach(P1 in SP,[PL], dec(P1,perm) &
      PL = dangerous,
      pl(P1,PL) & dec(PL,permLevel)
    )
  ) &

  applyTo(RR,IC,Cmp) &
  neg(isCProvider(Cmp)) &
  inApp(Cmp,A,S) &
  [IC,I] in SSInt &
  intType(I,intBroadcast) &
  foreach(SI in SSI,[A1],dec(SI,sysImgApp) &
    A1 neq A_,
    idSI(SI,A1) & dec(A1,idApp)
  ) &
  dec(P,perm) &
  (pl(P,signature) or pl(P,signatureOrSys)) &
  brperm(I,permc(P)) & 
  appCert(A,C,S) &
  certOfDefiner(P,C,S) & 
  appCert(A_,C_,S) &
  neg(certOfDefiner(P,C_,S)) &
  C_ neq cert:manufacturerCert &

  pre_receiveIntent1(SystemPerm,VulnerableSdk,IntentActionType,I,IC,A_,S).

:- dec_p_type(eavesdropping21).
eavesdropping21 :-
  dec(IC,iCmp) & dec(I,intent) & dec(Cmp,cmp) &
  dec([A,A_],idApp) & dec([C,C_],cert) &
  dec(S,system) & dec(SystemPerm,set(perm)) & 
  dec(VulnerableSdk,int) & dec(IntentActionType,rel(intent,pType)) &

  state(S,St) & dec(St,state) &
  running(St,RR) & dec(RR,rel(iCmp,cmp)) &
  sentIntents(St,SSInt) & dec(SSInt,set([iCmp,intent])) &
  environment(S,E) & dec(E,environment) &
  systemImage(E,SSI) & dec(SSI,set(sysImgApp)) &

% axiom onlyDangInPerms
delay(
  perms(St,PR) & dec(PR,rel(idApp,set(perm))) &
  foreach([X,SP] in PR,dec(X,idApp) & dec(SP,set(perm)) &
    foreach(P1 in SP,[PL], dec(P1,perm) &
      PL = dangerous,
      pl(P1,PL) & dec(PL,permLevel)
    )
  )
, false) &

  applyTo(RR,IC,Cmp) &
  neg(isCProvider(Cmp)) &
  inApp(Cmp,A,S) &
  [IC,I] in SSInt &
  intType(I,intBroadcast) &
  foreach(SI in SSI,[A1],dec(SI,sysImgApp) &
    A1 neq A_,
    idSI(SI,A1) & dec(A1,idApp)
  ) &
  dec(P,perm) &
  (pl(P,signature) or pl(P,signatureOrSys)) &
  intType(I,intBroadcast) &
  brperm(I,permc(P)) & 
  appCert(A,C,S) &
  certOfDefiner(P,C,S) & 
  appCert(A_,C_,S) &
  neg(certOfDefiner(P,C_,S)) &
  C_ neq cert:manufacturerCert &

  pre_receiveIntent21(SystemPerm,VulnerableSdk,IntentActionType,I,IC,A_,S).

:- dec_p_type(eavesdropping22).
eavesdropping22 :-
  dec(IC,iCmp) & dec(I,intent) & dec(Cmp,cmp) &
  dec([A,A_],idApp) & dec([C,C_],cert) &
  dec(S,system) & dec(SystemPerm,set(perm)) & 
  dec(VulnerableSdk,int) & dec(IntentActionType,rel(intent,pType)) &

  state(S,St) & dec(St,state) &
  running(St,RR) & dec(RR,rel(iCmp,cmp)) &
  sentIntents(St,SSInt) & dec(SSInt,set([iCmp,intent])) &
  environment(S,E) & dec(E,environment) &
  systemImage(E,SSI) & dec(SSI,set(sysImgApp)) &

% axiom onlyDangInPerms
delay(
  perms(St,PR) & dec(PR,rel(idApp,set(perm))) &
  foreach([X,SP] in PR,dec(X,idApp) & dec(SP,set(perm)) &
    foreach(P1 in SP,[PL], dec(P1,perm) &
      PL = dangerous,
      pl(P1,PL) & dec(PL,permLevel)
    )
  )
, false) &

  statesConsistency43(S) &

  applyTo(RR,IC,Cmp) &
  neg(isCProvider(Cmp)) &
  inApp(Cmp,A,S) &
  [IC,I] in SSInt &
  intType(I,intBroadcast) &
  foreach(SI in SSI,[A1],dec(SI,sysImgApp) &
    A1 neq A_,
    idSI(SI,A1) & dec(A1,idApp)
  ) &
  dec(P,perm) &
  (pl(P,signature) or pl(P,signatureOrSys)) &
  intType(I,intBroadcast) &
  brperm(I,permc(P)) & 
  appCert(A,C,S) &
  certOfDefiner(P,C,S) & 
  appCert(A_,C_,S) &
  neg(certOfDefiner(P,C_,S)) &
  C_ neq cert:manufacturerCert &

  pre_receiveIntent22(SystemPerm,VulnerableSdk,IntentActionType,I,IC,A_,S).

:- dec_p_type(eavesdropping3).
eavesdropping3 :-
  dec(IC,iCmp) & dec(I,intent) & dec(Cmp,cmp) &
  dec([A,A_],idApp) & dec([C,C_],cert) &
  dec(S,system) & dec(SystemPerm,set(perm)) & 
  dec(VulnerableSdk,int) & dec(IntentActionType,rel(intent,pType)) &

  state(S,St) & dec(St,state) &
  running(St,RR) & dec(RR,rel(iCmp,cmp)) &
  sentIntents(St,SSInt) & dec(SSInt,set([iCmp,intent])) &
  environment(S,E) & dec(E,environment) &
  systemImage(E,SSI) & dec(SSI,set(sysImgApp)) &

% axiom onlyDangInPerms
delay(
  perms(St,PR) & dec(PR,rel(idApp,set(perm))) &
  foreach([X,SP] in PR,dec(X,idApp) & dec(SP,set(perm)) &
    foreach(P1 in SP,[PL], dec(P1,perm) &
      PL = dangerous,
      pl(P1,PL) & dec(PL,permLevel)
    )
  )
, false) &

  applyTo(RR,IC,Cmp) &
  neg(isCProvider(Cmp)) &
  inApp(Cmp,A,S) &
  [IC,I] in SSInt &
  intType(I,intBroadcast) &
  foreach(SI in SSI,[A1],dec(SI,sysImgApp) &
    A1 neq A_,
    idSI(SI,A1) & dec(A1,idApp)
  ) &
  dec(P,perm) &
  (pl(P,signature) or pl(P,signatureOrSys)) &
  brperm(I,permc(P)) & 
  appCert(A,C,S) &
  certOfDefiner(P,C,S) & 
  appCert(A_,C_,S) &
  neg(certOfDefiner(P,C_,S)) &
  C_ neq cert:manufacturerCert &

  pre_receiveIntent3(SystemPerm,VulnerableSdk,IntentActionType,I,IC,A_,S).

:- dec_p_type(eavesdropping4).
eavesdropping4 :-
  dec(IC,iCmp) & dec(I,intent) & dec(Cmp,cmp) &
  dec([A,A_],idApp) & dec([C,C_],cert) &
  dec(S,system) & dec(SystemPerm,set(perm)) & 
  dec(VulnerableSdk,int) & dec(IntentActionType,rel(intent,pType)) &

  state(S,St) & dec(St,state) &
  running(St,RR) & dec(RR,rel(iCmp,cmp)) &
  sentIntents(St,SSInt) & dec(SSInt,set([iCmp,intent])) &
  environment(S,E) & dec(E,environment) &
  systemImage(E,SSI) & dec(SSI,set(sysImgApp)) &

% axiom onlyDangInPerms
delay(
  perms(St,PR) & dec(PR,rel(idApp,set(perm))) &
  foreach([X,SP] in PR,dec(X,idApp) & dec(SP,set(perm)) &
    foreach(P1 in SP,[PL], dec(P1,perm) &
      PL = dangerous,
      pl(P1,PL) & dec(PL,permLevel)
    )
  )
, false) &

  applyTo(RR,IC,Cmp) &
  neg(isCProvider(Cmp)) &
  inApp(Cmp,A,S) &
  [IC,I] in SSInt &
  intType(I,intBroadcast) &
  foreach(SI in SSI,[A1],dec(SI,sysImgApp) &
    A1 neq A_,
    idSI(SI,A1) & dec(A1,idApp)
  ) &
  dec(P,perm) &
  (pl(P,signature) or pl(P,signatureOrSys)) &
  brperm(I,permc(P)) & 
delay(
  appCert(A,C,S) &
  certOfDefiner(P,C,S)
, false) &
  appCert(A_,C_,S) &
  neg(certOfDefiner(P,C_,S)) &
  C_ neq cert:manufacturerCert &

  pre_receiveIntent4(SystemPerm,VulnerableSdk,IntentActionType,I,IC,A_,S).

:- dec_p_type(eavesdropping5).
eavesdropping5 :-
  dec(IC,iCmp) & dec(I,intent) & dec(Cmp,cmp) &
  dec([A,A_],idApp) & dec([C,C_],cert) &
  dec(S,system) & dec(SystemPerm,set(perm)) & 
  dec(VulnerableSdk,int) & dec(IntentActionType,rel(intent,pType)) &

  state(S,St) & dec(St,state) &
  running(St,RR) & dec(RR,rel(iCmp,cmp)) &
  sentIntents(St,SSInt) & dec(SSInt,set([iCmp,intent])) &
  environment(S,E) & dec(E,environment) &
  systemImage(E,SSI) & dec(SSI,set(sysImgApp)) &

% axiom onlyDangInPerms
delay(
  perms(St,PR) & dec(PR,rel(idApp,set(perm))) &
  foreach([X,SP] in PR,dec(X,idApp) & dec(SP,set(perm)) &
    foreach(P1 in SP,[PL], dec(P1,perm) &
      PL = dangerous,
      pl(P1,PL) & dec(PL,permLevel)
    )
  )
, false) &

  applyTo(RR,IC,Cmp) &
  neg(isCProvider(Cmp)) &
  inApp(Cmp,A,S) &
  [IC,I] in SSInt &
  intType(I,intBroadcast) &
  foreach(SI in SSI,[A1],dec(SI,sysImgApp) &
    A1 neq A_,
    idSI(SI,A1) & dec(A1,idApp)
  ) &
  dec(P,perm) &
  (pl(P,signature) or pl(P,signatureOrSys)) &
  brperm(I,permc(P)) & 
delay(
  appCert(A,C,S) &
  certOfDefiner(P,C,S)
, false) &
  appCert(A_,C_,S) &
  neg(certOfDefiner(P,C_,S)) &
  C_ neq cert:manufacturerCert &

  pre_receiveIntent5(SystemPerm,VulnerableSdk,IntentActionType,I,IC,A_,S).

:- dec_p_type(intentSpoofing1).
intentSpoofing1 :-
  dec([A1,A2],idApp) & dec([C1,C2],cmp) & dec(IC1,iCmp) &
  dec(I,intent) & dec(S,system) &
  dec(SystemPerm,set(perm)) & 
  dec(VulnerableSdk,int) & dec(IntentActionType,rel(intent,pType)) &

  state(S,St) & dec(St,state) &
  running(St,RR) & dec(RR,rel(iCmp,cmp)) &

    statesConsistency1(S) &
    notRepeatedCmps1(S) &
    notRepeatedCmps2(S) &
    notRepeatedCmps3(S) &

    inApp(C1,A1,S) &
    inApp(C2,A2,S) &
    A1 neq A2 &
    applyTo(RR,IC1,C1) &
    delay(
      neg(canBeStarted(C2,S)) &
      neg(isCProvider(C1)) &
      getCmpId(C2,I1) & dec(I1,idCmp) &
      cmpName(I,idcmp(I1)),
      false
    ) &
    pre_receiveIntent6(SystemPerm,VulnerableSdk,IntentActionType,I,IC1,A2,S).

% takes too long
:- dec_p_type(intentSpoofing2).
intentSpoofing2 :-
  dec([A1,A2],idApp) & dec([C1,C2],cmp) & dec(IC1,iCmp) &
  dec(I,intent) & dec(S,system) &
  dec(SystemPerm,set(perm)) & 
  dec(VulnerableSdk,int) & dec(IntentActionType,rel(intent,pType)) &

  state(S,St) & dec(St,state) &
  running(St,RR) & dec(RR,rel(iCmp,cmp)) &

  statesConsistency1(S) &
  allCmpDifferent1(S) &
  allCmpDifferent2(S) &
  allCmpDifferent3(S) &
  allCmpDifferent4(S) &
  allCmpDifferent5(S) &
  notRepeatedCmps1(S) &
  notRepeatedCmps2(S) &
  notRepeatedCmps3(S) &
  notDupApp(S) &
  notDupSysApp(S) &

  inApp(C2,A2,S) &
  applyTo(RR,IC1,C1) &
  getCmpId(C2,I1) & dec(I1,idCmp) &
  cmpName(I,idcmp(I1)) &
  neg(canBeStarted(C2,S)) &
  delay(
    inApp(C1,A1,S) &
    A1 neq A2 &
    neg(isCProvider(C1)) &
    applyTo(RR,IC1,C1),
    false
  ) &
  pre_receiveIntent7(SystemPerm,VulnerableSdk,IntentActionType,I,IC1,A2,S).

:- dec_p_type(check_prop1).
check_prop1 :-
  dec([NPO,PO],int) &
  prolog_call((get_time(Ti),nb_setval(po,0))) &
  (nl & write('Checking dangerousPermMissing...') &
     prolog_call((nb_getval(po,PO),flush_output)) & NPO is PO+1 &
     prolog_call(once(setlog(dangerousPermMissing))) &
     writedone & prolog_call((nb_setval(po,NPO),flush_output,fail)) or
   nl & write('Checking delegateGrantPRevoke...') &
     prolog_call((nb_getval(po,PO),flush_output)) & NPO is PO+1 &
     prolog_call(once(setlog(delegateGrantPRevoke))) &
     writedone & prolog_call((nb_setval(po,NPO),flush_output,fail)) or
   nl & write('Checking cannotAutoGrantWithoutGroup...') &
     prolog_call((nb_getval(po,PO),flush_output)) & NPO is PO+1 &
     naf(cannotAutoGrantWithoutGroup) &
     writedone & prolog_call((nb_setval(po,NPO),flush_output,fail)) or
   nl & write('Checking execAutoGrantWithoutIndividualPerms...') &
     prolog_call((nb_getval(po,PO),flush_output)) & NPO is PO+1 &
     prolog_call(once(setlog(execAutoGrantWithoutIndividualPerms))) &
     writedone & prolog_call((nb_setval(po,NPO),flush_output,fail)) or
   nl & write('Checking dangerousPermissionAutoGranted...') &
     prolog_call((nb_getval(po,PO),flush_output)) & NPO is PO+1 &
     prolog_call(once(setlog(dangerousPermissionAutoGranted))) &
     writedone & prolog_call((nb_setval(po,NPO),flush_output,fail)) or
   nl & write('Checking revokeGroupRevokesIndividualPerms...') &
     prolog_call((nb_getval(po,PO),flush_output)) & NPO is PO+1 &
     naf(revokeGroupRevokesIndividualPerms) &
     writedone & prolog_call((nb_setval(po,NPO),flush_output,fail)) or
   nl & write('Checking notVerifiedOldAppCantReceive...') &
     prolog_call((nb_getval(po,PO),flush_output)) & NPO is PO+1 &
     naf(notVerifiedOldAppCantReceive) &
     writedone & prolog_call((nb_setval(po,NPO),flush_output,fail)) or
   nl & write('Checking sacProtectedWithNormalPerms...') &
     prolog_call((nb_getval(po,PO),flush_output)) & NPO is PO+1 &
     prolog_call(once(setlog(sacProtectedWithNormalPerms))) &
     writedone & prolog_call((nb_setval(po,NPO),flush_output,fail)) or
   nl & write('Checking eavesdropping1...') &
     prolog_call((nb_getval(po,PO),flush_output)) & NPO is PO+1 &
     naf(eavesdropping1) &
     writedone & prolog_call((nb_setval(po,NPO),flush_output,fail)) or
   nl & write('Checking eavesdropping21...') &
     prolog_call((nb_getval(po,PO),flush_output)) & NPO is PO+1 &
     naf(eavesdropping21) &
     writedone & prolog_call((nb_setval(po,NPO),flush_output,fail)) or
   nl & write('Checking eavesdropping22...') &
     prolog_call((nb_getval(po,PO),flush_output)) & NPO is PO+1 &
     naf(eavesdropping22) &
     writedone & prolog_call((nb_setval(po,NPO),flush_output,fail)) or
   nl & write('Checking eavesdropping3...') &
     prolog_call((nb_getval(po,PO),flush_output)) & NPO is PO+1 &
     naf(eavesdropping3) &
     writedone & prolog_call((nb_setval(po,NPO),flush_output,fail)) or
   nl & write('Checking eavesdropping4...') &
     prolog_call((nb_getval(po,PO),flush_output)) & NPO is PO+1 &
     naf(eavesdropping4) &
     writedone & prolog_call((nb_setval(po,NPO),flush_output,fail)) or
   nl & write('Checking eavesdropping5...') &
     prolog_call((nb_getval(po,PO),flush_output)) & NPO is PO+1 &
     naf(eavesdropping5) &
     writedone & prolog_call((nb_setval(po,NPO),flush_output,fail)) or
   nl & write('Checking intentSpoofing1...') &
     prolog_call((nb_getval(po,PO),flush_output)) & NPO is PO+1 &
     naf(intentSpoofing1) &
     writedone & prolog_call((nb_setval(po,NPO),flush_output,fail)) or
   prolog_call((get_time(Te),nb_getval(po,PO),T is Te - Ti)) &
     nl & nl & write(PO) &
     write(' proof obligations (out of 15) discharged in ') &
     write(T) & write(' seconds') & nl
  ).

:- dec_p_type(check_prop).
check_prop :-
  check_prop1 &
  check_revokeAndNotGrant &
  check_ifOldAppRunThenVerified.

